import { reactive, computed, toRefs } from 'vue';

/**
 * @public
 */
function usePagination({ currentPage, perPageSize, totalSize, }) {
    validatePageNumber('currentPage', currentPage.value);
    validatePageNumber('perPageSize', perPageSize.value);
    validatePageNumber('totalSize', totalSize.value);
    const state = reactive({
        currentPage,
        perPageSize,
        totalSize,
        currentPerPageSize: computed(() => state.currentEndSize - state.currentStartSize + 1),
        totalPageSize: computed(() => calcTotalPageSize(perPageSize.value, totalSize.value)),
        pageRange: computed(() => createRange(state.totalPageSize)),
        currentStartSize: computed(() => state.prevPage * state.perPageSize + 1),
        currentEndSize: computed(() => (state.isLastPage ? state.totalSize : state.currentPage * state.perPageSize)),
        isFirstPage: computed(() => state.firstPage === state.currentPage),
        firstPage: 1,
        isLastPage: computed(() => state.lastPage === state.currentPage),
        lastPage: computed(() => state.totalPageSize),
        hasPrevPage: computed(() => state.currentPage > 1),
        prevPage: computed(() => currentPage.value - 1),
        hasNextPage: computed(() => state.lastPage > state.currentPage),
        nextPage: computed(() => currentPage.value + 1),
    });
    return toRefs(state);
}
/**
 * @public
 */
function useGuaranteePageSize(currentPage, totalPageSize) {
    if (currentPage.value < 1)
        currentPage.value = 1;
    if (currentPage.value > totalPageSize.value)
        currentPage.value = totalPageSize.value;
    if (Number.isInteger(currentPage.value) === false)
        currentPage.value = 1;
}
/**
 * @public
 */
function validatePageNumber(name, number) {
    if (Number.isInteger(number) === false)
        throw TypeError(`\`${name}\` must be an integer.`);
    if (Number.isSafeInteger(number) === false)
        throw RangeError(`\`${name}\` must be a safe integer.`);
    if (number <= 0)
        throw RangeError(`\`${name}\` must greater than 0.`);
}
/**
 * @public
 */
function calcTotalPageSize(perPageSize, totalSize) {
    return Math.ceil(totalSize / perPageSize);
}
/**
 * @public
 */
function isValidPageSize(currentPage, totalPageSize) {
    return Number.isInteger(currentPage.value) && currentPage.value >= 1 && currentPage.value <= totalPageSize.value;
}
/**
 * @public
 */
function createRange(last, first = 1) {
    return Array(last - first + 1)
        .fill(null)
        .map((_, index) => index + first);
}

export { calcTotalPageSize, createRange, isValidPageSize, useGuaranteePageSize, usePagination, validatePageNumber };
